@@ -39,21 +39,45 @@ Utilities for dealing with Signed JSON Web Tokens.
 
 """
 
-import os
 import struct
 import hashlib
-from M2Crypto import RSA as _RSA
-from vep.m2_dsa_patch import DSA as _DSA
+from binascii import unhexlify
+from vep._m2_monkeypatch import DSA as _DSA
+from vep._m2_monkeypatch import RSA as _RSA
 
 from vep.utils import decode_bytes, encode_bytes
 from vep.utils import decode_json_bytes, encode_json_bytes
 
 
+def parse(jwt, cls=None):
+    """Parse a JWT from a string."""
+    if cls is None:
+        cls = JWT
+    algorithm, payload, signature = jwt.split(".")
+    signed_data = algorithm + "." + payload
+    try:
+        algorithm = decode_json_bytes(algorithm)["alg"]
+    except KeyError:
+        raise ValueError("badly formed JWT")
+    payload = decode_json_bytes(payload)
+    signature = decode_bytes(signature)
+    return cls(algorithm, payload, signature, signed_data)
+
+
+def generate(payload, key):
+    """Generate and sign a JWT for a dict payload."""
+    alg = key.__class__.__name__[:-3]
+    algorithm = encode_json_bytes({"alg": alg})
+    payload = encode_json_bytes(payload)
+    signature = encode_bytes(key.sign(".".join((algorithm, payload))))
+    return ".".join((algorithm, payload, signature))
+
+
 class JWT(object):
     """Class for parsing signed JSON Web Tokens.
 
-    To parse a JWT from a bytestring, use JTW.parse(data).  The default
-    constructor is only for internal purposes.
+    To parse a JWT from a bytestring, use the module-level parse() function.
+    This class is really only for internal purposes.
     """
 
     def __init__(self, algorithm, payload, signature, signed_data):
@@ -62,28 +86,6 @@ class JWT(object):
         self.signature = signature
         self.signed_data = signed_data
 
-    @classmethod
-    def parse(cls, jwt):
-        """Parse a JWT from a string."""
-        algorithm, payload, signature = jwt.split(".")
-        signed_data = algorithm + "." + payload
-        try:
-            algorithm = decode_json_bytes(algorithm)["alg"]
-        except KeyError:
-            raise ValueError("badly formed JWT")
-        payload = decode_json_bytes(payload)
-        signature = decode_bytes(signature)
-        return cls(algorithm, payload, signature, signed_data)
-
-    @classmethod
-    def generate(cls, payload, key):
-        """Generate and sign a JWT for a dict payload."""
-        alg = key.__class__.__name__[:-3]
-        algorithm = encode_json_bytes({"alg": alg})
-        payload = encode_json_bytes(payload)
-        signature = encode_bytes(key.sign(".".join((algorithm, payload))))
-        return ".".join((algorithm, payload, signature))
-
     def check_signature(self, key_data):
         """Check that the JWT was signed with the given key."""
         if not self.algorithm.startswith(key_data["algorithm"]):
@@ -109,10 +111,10 @@ class Key(object):
     """Generic base class for Key objects."""
 
     def verify(self, signed_data, signature):
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: nocover
 
     def sign(self, data):
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: nocover
 
 
 #
@@ -127,7 +129,12 @@ class RSKey(object):
     def __init__(self, data):
         e = int2mpint(int(data["e"]))
         n = int2mpint(int(data["n"]))
-        self.rsa = _RSA.new_pub_key((e, n))
+        try:
+            d = int2mpint(int(data["d"]))
+        except KeyError:
+            self.rsa = _RSA.new_pub_key((e, n))
+        else:
+            self.rsa = _RSA.new_key((e, n, d))
 
     def verify(self, signed_data, signature):
         digest = self.HASHMOD(signed_data).digest()
@@ -136,6 +143,10 @@ class RSKey(object):
         except _RSA.RSAError:
             return False
 
+    def sign(self, data):
+        digest = self.HASHMOD(data).digest()
+        return self.rsa.sign(digest, self.HASHNAME)
+
 
 class RS64Key(RSKey):
     SIZE = 64
@@ -156,8 +167,8 @@ class RS256Key(RSKey):
 
 
 #
-#  DSA keys, implemented by hand because I haven't figured out how to
-#  map what M2Crypto provides onto what the browserid.org server does.
+#  DSA keys, implemented using the DSA support in M2Crypto, along with
+#  some formatting tweaks to match what the browserid node-js server does.
 #
 
 class DSKey(object):
@@ -173,11 +184,11 @@ class DSKey(object):
         if "x" not in data:
             self.x = None
             self.dsa = _DSA.load_pub_key_params(int2mpint(p), int2mpint(q),
-	                                        int2mpint(g), int2mpint(y))
+                                                int2mpint(g), int2mpint(y))
         else:
             self.x = x = long(data["x"], 16)
             self.dsa = _DSA.load_key_params(int2mpint(p), int2mpint(q),
-	                                    int2mpint(g), int2mpint(y),
+                                            int2mpint(g), int2mpint(y),
                                             int2mpint(x))
 
     def verify(self, signed_data, signature):
@@ -227,15 +238,16 @@ class DS256Key(DSKey):
 
 
 def int2mpint(x):
-    """Convert a Python integer into a string in OpenSSL's MPINT format."""
-    # The horror...the horror...
-    bytes = []
-    while x:
-        bytes.append(chr(x % 256))
-        x = x / 256
-    # Add an extra significant byte that's all zeros.  This helps to ensure
-    # that the resulting bignum always has the correct number of significant
-    # bits.  I don't understand why, but it does.
-    bytes.append("\x00")
-    bytes.reverse()
-    return struct.pack(">I", len(bytes)) + "".join(bytes)
+    """Convert a Python long integer to a string in OpenSSL's MPINT format."""
+    # MPINT is big-endian bytes with a size prefix.
+    # It's faster to go via hex encoding in C code than it is to try
+    # encoding directly into binary with a python-level loop.
+    # (and hex-slice-strip seems consistently faster than using "%x" format)
+    hexbytes = hex(x)[2:].rstrip("L")
+    if len(hexbytes) % 2:
+        hexbytes = "0" + hexbytes
+    bytes = unhexlify(hexbytes)
+    # Add an extra significant byte that's just zero.  I think this is only
+    # necessary if the number has its MSB, to prevent it being mistaken for
+    # a sign bit.  I do it uniformly since it's simpler and still correct.
+    return struct.pack(">I", len(bytes)+1) + "\x00" + bytes
