@@ -39,13 +39,16 @@ import json
 import unittest
 import warnings
 
+import vep
+from vep import jwt
 from vep import RemoteVerifier, LocalVerifier, DummyVerifier
-from vep.utils import encode_json_bytes, encode_bytes, decode_json_bytes
-from vep.jwt import JWT, load_key
+from vep.verifiers.local import FIFOCache
+from vep.utils import encode_json_bytes, decode_json_bytes
 from vep.errors import (TrustError,
                         ConnectionError,
                         ExpiredSignatureError,
                         InvalidSignatureError,
+                        InvalidIssuerError,
                         AudienceMismatchError)
 
 # This is an old assertion I generated on myfavoritebeer.org.
@@ -129,7 +132,7 @@ class VerifierTestCases(object):
         # This one has no certificates
         pub, priv = DummyVerifier._get_keypair("TEST")
         assertion = encode_json_bytes({
-            "assertion": JWT.generate({"aud": "TEST"}, priv),
+            "assertion": jwt.generate({"aud": "TEST"}, priv),
             "certificates": []
         })
         self.assertRaises(errors, self.verifier.verify, assertion)
@@ -138,70 +141,74 @@ class VerifierTestCases(object):
 class TestLocalVerifier(unittest.TestCase, VerifierTestCases):
 
     def setUp(self):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("default")
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
             self.verifier = LocalVerifier()
+        # There should be a warning about using this verifier.
+        self.assertEquals(w[0].category, FutureWarning)
 
     def test_error_while_fetching_public_key(self):
-        def fetch_public_key(hostname):
+        def urlopen(*args, **kwds):
             raise RuntimeError("TESTING")
-        self.verifier.fetch_public_key = fetch_public_key
+        self.verifier.urlopen = urlopen
         self.assertRaises(ConnectionError,
                           self.verifier.verify, EXPIRED_ASSERTION, now=0)
 
-    def test_missing_host_meta_document(self):
+    def test_missing_well_known_document(self):
         def urlopen(url, data):
             raise RuntimeError("404 Not Found")
         self.verifier.urlopen = urlopen
-        self.assertRaises(ConnectionError,
+        self.assertRaises(InvalidIssuerError,
                           self.verifier.verify, EXPIRED_ASSERTION, now=0)
 
-    def test_malformed_host_meta_document(self):
+    def test_malformed_well_known_document(self):
         def urlopen(url, data):
             class response(object):
                 @staticmethod
                 def read():
-                    return "I AINT NO XML, FOOL!"
+                    return "I AINT NO JSON, FOOL!"
             return response
         self.verifier.urlopen = urlopen
-        self.assertRaises(ConnectionError,
+        self.assertRaises(InvalidIssuerError,
                           self.verifier.verify, EXPIRED_ASSERTION, now=0)
 
-    def test_host_meta_with_no_key_link(self):
-        def urlopen(url, data):
-            class response(object):
-                @staticmethod
-                def read():
-                    return "<Meta>"\
-                           " <Link rel='not the key link' href='haha' />"\
-                           "</Meta>"
-            return response
-        self.verifier.urlopen = urlopen
-        self.assertRaises(ConnectionError,
-                          self.verifier.verify, EXPIRED_ASSERTION, now=0)
-
-    def test_host_meta_with_key_link(self):
-        #  The browserid.org server doesn't currently have host-meta.
-        #  This simulates it with a link to the known public key URL.
+    def test_malformed_pub_key_document(self):
         called = []
         def urlopen(url, data):
-            #  If already called, return the mock key.
-            if called:
-                class response(object):
-                    @staticmethod
-                    def read():
-                        key = DummyVerifier.fetch_public_key("browserid.org")
-                        return json.dumps(key)
-            #  Otherwise, return the necessary XML to link to it.
-            else:
-                class response(object):
-                    @staticmethod
-                    def read():
-                        rel = self.verifier.HOST_META_REL_PUBKEY
-                        return "<Meta>"\
-                               " <Link rel='" + rel + "' href='haha' />"\
-                               "</Meta>"
-            called.append(True)
+            # First call must raise 404 so it will look for /pk.
+            # Second call must return invalid JSON.
+            class response(object):
+                @staticmethod
+                def read():
+                    if not called:
+                        called.append(True)
+                        raise ValueError("404 Not Found")
+                    return "I AINT NO JSON, FOOL!"
+            return response
+        self.verifier.urlopen = urlopen
+        self.assertRaises(InvalidIssuerError,
+                          self.verifier.verify, EXPIRED_ASSERTION, now=0)
+
+    def test_well_known_doc_with_no_public_key(self):
+        def urlopen(url, data):
+            class response(object):
+                @staticmethod
+                def read():
+                    return "{}"
+            return response
+        self.verifier.urlopen = urlopen
+        self.assertRaises(InvalidIssuerError,
+                          self.verifier.verify, EXPIRED_ASSERTION, now=0)
+
+    def test_well_known_doc_with_public_key(self):
+        #  The browserid.org server doesn't currently have /.well-known/vep.
+        #  This simulates it with a dummy key.
+        def urlopen(url, data):  # NOQA
+            class response(object):
+                @staticmethod
+                def read():
+                    key = DummyVerifier.fetch_public_key("browserid.org")
+                    return json.dumps({"public-key": key})
             return response
         self.verifier.urlopen = urlopen
         assertion = DummyVerifier.make_assertion("t@m.com", "http://e.com")
@@ -213,7 +220,7 @@ class TestLocalVerifier(unittest.TestCase, VerifierTestCases):
                 @staticmethod
                 def info():
                     return {"Content-Length": "forty-two"}
-                @staticmethod
+                @staticmethod  # NOQA
                 def read(size):
                     raise RuntimeError  # pragma: nocover
             return response
@@ -225,7 +232,7 @@ class TestLocalVerifier(unittest.TestCase, VerifierTestCases):
         self.assertRaises(ValueError,
                           self.verifier.verify_certificate_chain, [])
         certs = decode_json_bytes(EXPIRED_ASSERTION)["certificates"]
-        certs = [JWT.parse(cert) for cert in certs]
+        certs = [jwt.parse(cert) for cert in certs]
         self.assertRaises(ExpiredSignatureError,
                           self.verifier.verify_certificate_chain, certs)
 
@@ -254,7 +261,7 @@ class TestRemoteVerifier(unittest.TestCase, VerifierTestCases):
                 @staticmethod
                 def info():
                     return {"Content-Length": "forty-two"}
-                @staticmethod
+                @staticmethod  # NOQA
                 def read(size):
                     raise RuntimeError  # pragma: nocover
             return response
@@ -304,9 +311,11 @@ class TestRemoteVerifier(unittest.TestCase, VerifierTestCases):
 class TestDummyVerifier(unittest.TestCase, VerifierTestCases):
 
     def setUp(self):
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter("default")
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
             self.verifier = DummyVerifier()
+        # There should be no warnings from the dummy verifier.
+        self.assertEquals(len(w), 0)
 
     def test_verification_of_valid_dummy_assertion(self):
         audience = "http://example.com"
@@ -316,6 +325,15 @@ class TestDummyVerifier(unittest.TestCase, VerifierTestCases):
         self.assertRaises(AudienceMismatchError,
                           self.verifier.verify, assertion, "http://moz.com")
 
+    def test_verification_of_oldstyle_dummy_assertion(self):
+        audience = "http://example.com"
+        assertion = self.verifier.make_assertion("test@example.com", audience,
+                                                 new_style=False)
+        self.assertTrue(self.verifier.verify(assertion))
+        self.assertTrue(self.verifier.verify(assertion, audience))
+        self.assertRaises(AudienceMismatchError,
+                          self.verifier.verify, assertion, "http://moz.com")
+
     def test_verification_of_untrusted_issuer(self):
         audience = "http://example.com"
         issuer = "moz.com"
@@ -351,3 +369,77 @@ class TestDummyVerifier(unittest.TestCase, VerifierTestCases):
                                                  certificate_sig="CORRUPTUS")
         self.assertRaises(InvalidSignatureError,
                           self.verifier.verify, assertion)
+
+    def test_cache_eviction_based_on_time(self):
+        cache = FIFOCache(cache_timeout=0.1)
+        verifier = DummyVerifier(cache=cache)
+        # Prime the cache by verifying an assertion.
+        assertion = self.verifier.make_assertion("test@example.com", "")
+        self.assertTrue(verifier.verify(assertion))
+        # Make it error out if re-fetching the keys
+        def fetch_public_key(hostname):
+            raise RuntimeError("key fetch disabled")
+        verifier.fetch_public_key = fetch_public_key
+        # It should be in the cache, so this works fine.
+        verifier.verify(assertion)
+        # But after sleeping it gets evicted and the error is triggered.
+        time.sleep(0.1)
+        self.assertRaises(RuntimeError, verifier.verify, assertion)
+
+    def test_cache_eviction_based_on_size(self):
+        cache = FIFOCache(max_size=2)
+        verifier = DummyVerifier(cache=cache)
+        # Prime the cache by verifying some assertions.
+        assertion1 = self.verifier.make_assertion("test@1.com", "", "1.com")
+        self.assertTrue(verifier.verify(assertion1))
+        assertion2 = self.verifier.make_assertion("test@2.com", "", "2.com")
+        self.assertTrue(verifier.verify(assertion2))
+        self.assertEquals(len(cache), 2)
+        # Hitting a third host should evict the first public key.
+        assertion3 = self.verifier.make_assertion("test@3.com", "", "3.com")
+        self.assertTrue(verifier.verify(assertion3))
+        self.assertEquals(len(cache), 2)
+        # Make it error out if re-fetching any keys
+        def fetch_public_key(hostname):
+            raise RuntimeError("key fetch disabled")
+        verifier.fetch_public_key = fetch_public_key
+        # It should have to re-fetch for 1, but not 2.
+        self.assertTrue(verifier.verify(assertion2))
+        self.assertRaises(RuntimeError, verifier.verify, assertion1)
+
+    def test_cache_eviction_during_write(self):
+        cache = FIFOCache(cache_timeout=0.1)
+        verifier = DummyVerifier(cache=cache)
+        # Prime the cache by verifying an assertion.
+        assertion1 = self.verifier.make_assertion("test@1.com", "", "1.com")
+        self.assertTrue(verifier.verify(assertion1))
+        self.assertEquals(len(cache), 1)
+        # Let that cached key expire
+        time.sleep(0.1)
+        # Now grab a different key; caching it should purge the expired key.
+        assertion2 = self.verifier.make_assertion("test@2.com", "", "2.com")
+        self.assertTrue(verifier.verify(assertion2))
+        self.assertEquals(len(cache), 1)
+        # Check that only the second entry is in cache.
+        def fetch_public_key(hostname):
+            raise RuntimeError("key fetch disabled")
+        verifier.fetch_public_key = fetch_public_key
+        self.assertTrue(verifier.verify(assertion2))
+        self.assertRaises(RuntimeError, verifier.verify, assertion1)
+
+
+class TestShortcutFunctions(unittest.TestCase):
+
+    def test_shortcut(self):
+        self.assertRaises(TrustError, vep.verify, EXPIRED_ASSERTION)
+
+    def test_shortcut_remote(self):
+        self.assertRaises(TrustError, vep.verify_remote, EXPIRED_ASSERTION)
+
+    def test_shortcut_local(self):
+        with warnings.catch_warnings():
+            warnings.simplefilter("always")
+            self.assertRaises(TrustError, vep.verify_local, EXPIRED_ASSERTION)
+
+    def test_shortcut_dummy(self):
+        self.assertRaises(TrustError, vep.verify_dummy, EXPIRED_ASSERTION)
