@@ -47,6 +47,7 @@ import base64
 import socket
 import httplib
 import urllib2
+import warnings
 from fnmatch import fnmatch
 
 from vep.errors import ConnectionError
@@ -59,7 +60,7 @@ def decode_bytes(value):
     meaning we can't use the stdlib routines to decode them directly.  This
     is a simple wrapper that adds the padding back in.
 
-    If the value is not correctly encoded, ValueErro will be raised.
+    If the value is not correctly encoded, ValueError will be raised.
     """
     if isinstance(value, unicode):
         value = value.encode("ascii")
@@ -108,6 +109,67 @@ def encode_json_bytes(obj):
     return encode_bytes(json.dumps(obj))
 
 
+def bundle_certs_and_assertion(certificates, assertion, new_style=True):
+    """Bundle certificates and assertion into a single string.
+
+    This function produces a VEP "bundled assertion" that combines the
+    certificate chain and final assertion into a single string.  By default
+    it uses the "new-style" tilde-separated format; pass new_style=False to
+    use the older b64-encoded-JSON format.
+    """
+    if new_style:
+        return "~".join(certificates) + "~" + assertion
+    else:
+        return encode_json_bytes({
+          "certificates": certificates,
+          "assertion": assertion,
+        })
+
+
+def unbundle_certs_and_assertion(bundle):
+    """Unbundle certificates and assertion from a single string.
+
+    This function parse a VEP "bundled assertion" into the contained chain
+    of certificates and final assertion.  The returned value is a tuple
+    (certificates, assertion).
+    """
+    if "~" in bundle:
+        certificates, assertion = bundle.rsplit("~", 1)
+        certificates = certificates.split("~")
+    else:
+        data = decode_json_bytes(bundle)
+        certificates = data["certificates"]
+        assertion = data["assertion"]
+    return certificates, assertion
+
+
+def get_assertion_info(assertion):
+    """Parse interesting information out of a BrowserID assertion.
+
+    This function decodes and parses the given BrowserID assertion, returning
+    a dict with the following items:
+
+       * principal:  the asserted identity, eg: {"email": "test@example.com"}
+       * audience:   the audience to whom it is asserted
+
+    This does *not* verify the assertion at all, it is merely a way to see
+    the information that is being asserted.  If the assertion is malformed
+    then ValueError will be raised.
+    """
+    info = {}
+    try:
+        certificates, assertion = unbundle_certs_and_assertion(assertion)
+        # Get the asserted principal out of the certificate chain.
+        payload = decode_json_bytes(certificates[-1].split(".")[1])
+        info["principal"] = payload["principal"]
+        # Get the audience out of the assertion token.
+        payload = decode_json_bytes(assertion.split(".")[1])
+        info["audience"] = payload["aud"]
+    except (TypeError, KeyError), e:
+        raise ValueError(str(e))
+    return info
+
+
 # When using secure_urlopen we search for the platform default ca-cert file.
 # This is done on-demand and the result cached in this global variable.
 DEFAULT_CACERT_FILE = None
@@ -119,7 +181,7 @@ POSSIBLE_CACERT_FILES = ["/etc/ssl/certs/ca-certificates.crt",
 _OPENER_CACHE = {}
 
 
-## We're using M2Crypto anyway, should also use it for the below?
+## TODO: We're using M2Crypto anyway, should also use it for the below?
 
 def secure_urlopen(url, data=None, timeout=None, ca_certs=None):
     """More secure replacement for urllib2.urlopen.
@@ -137,8 +199,12 @@ def secure_urlopen(url, data=None, timeout=None, ca_certs=None):
                     ca_certs = DEFAULT_CACERT_FILE = filenm
                     break
             if ca_certs is None:
-                err = "could not locate default ca_certs file"
-                raise ConnectionError(err)
+                msg = "Could not locate a CA certificates file for HTTPS."\
+                      " Your requests will be vulnerable to man-in-the-middle"\
+                      " attacks.  It is *HIGHLY RECOMMENDED* that you specify"\
+                      " the ca_certs parameter with the path to a valid"\
+                      " certificates file."
+                warnings.warn(msg, stacklevel=2)
     # Use a cached opener if possible.
     try:
         opener = _OPENER_CACHE[ca_certs]
@@ -195,8 +261,6 @@ class ValidatingHTTPSConnection(httplib.HTTPSConnection):
 
     def __init__(self, *args, **kwds):
         self.ca_certs = kwds.pop("ca_certs", None)
-        if self.ca_certs is None:
-            raise TypeError("missing keyword argument: ca_certs")
         httplib.HTTPSConnection.__init__(self, *args, **kwds)
 
     def connect(self):
@@ -205,11 +269,16 @@ class ValidatingHTTPSConnection(httplib.HTTPSConnection):
         if self._tunnel_host:
             self.sock = sock
             self._tunnel()
-        self.sock = ssl.wrap_socket(sock, ssl_version=ssl.PROTOCOL_SSLv3,
-                                    cert_reqs=ssl.CERT_REQUIRED,
-                                    ca_certs=self.ca_certs)
-        cert = self.sock.getpeercert()
-        self._validate_certificate(cert)
+        kwds = {"ssl_version": ssl.PROTOCOL_SSLv3}
+        if self.ca_certs is None:
+            kwds["cert_reqs"] = ssl.CERT_NONE
+        else:
+            kwds["ca_certs"] = self.ca_certs
+            kwds["cert_reqs"] = ssl.CERT_REQUIRED
+        self.sock = ssl.wrap_socket(sock, **kwds)
+        if self.ca_certs is not None:
+            cert = self.sock.getpeercert()
+            self._validate_certificate(cert)
 
     def _validate_certificate(self, cert):
         now = time.time()
