@@ -35,12 +35,11 @@
 # ***** END LICENSE BLOCK *****
 
 import time
-import json
 import hashlib
 
 from vep.verifiers.local import LocalVerifier
-from vep.utils import encode_bytes
-from vep.jwt import JWT, DS128Key
+from vep.utils import encode_bytes, bundle_certs_and_assertion
+from vep import jwt
 
 
 # These are values used to generate dummy DSA keys.
@@ -100,7 +99,8 @@ class DummyVerifier(LocalVerifier):
 
     @classmethod
     def make_assertion(cls, email, audience, issuer=None, exp=None,
-                       assertion_sig=None, certificate_sig=None):
+                       assertion_sig=None, certificate_sig=None,
+                       new_style=True):
         """Generate a new dummy assertion for the given email address.
 
         This method lets you generate VEP assertions using dummy private keys.
@@ -123,7 +123,7 @@ class DummyVerifier(LocalVerifier):
           "exp": exp,
           "aud": audience,
         }
-        assertion = JWT.generate(assertion, email_priv)
+        assertion = jwt.generate(assertion, email_priv)
         if assertion_sig is not None:
             assertion = ".".join(assertion.split(".")[:-1] +
                                  [encode_bytes(assertion_sig)])
@@ -135,16 +135,12 @@ class DummyVerifier(LocalVerifier):
           "principal": {"email": email},
           "public-key": email_pub,
         }
-        certificate = JWT.generate(certificate, iss_priv)
+        certificate = jwt.generate(certificate, iss_priv)
         if certificate_sig is not None:
             certificate = ".".join(certificate.split(".")[:-1] +
                                    [encode_bytes(certificate_sig)])
         # Combine them into a VEP bundled assertion.
-        data = {
-          "certificates": [certificate],
-          "assertion": assertion,
-        }
-        return encode_bytes(json.dumps(data))
+        return bundle_certs_and_assertion([certificate], assertion, new_style)
 
     @classmethod
     def _get_keypair(cls, hostname):
@@ -177,10 +173,10 @@ class DummyVerifier(LocalVerifier):
           "y": _hex(y),
           "x": _hex(x),
         }
-        privkey = DS128Key(data)
+        privkey = jwt.DS128Key(data)
         del data["x"]
         return data, privkey
 
 if __name__ == "__main__":
-    import sys
-    print DummyVerifier.make_assertion(*sys.argv[1:])
+    import sys  # pragma: nocover
+    print DummyVerifier.make_assertion(*sys.argv[1:])  # pragma: nocover
